@@ -63,7 +63,9 @@ class TMonitor(Thread):
     _sleep = None
 
     def __init__(self, tqdm_cls, sleep_interval):
-        sys.setcheckinterval(100)
+        # setcheckinterval is deprecated
+        getattr(sys, 'setswitchinterval',
+                getattr(sys, 'setcheckinterval'))(100)
         Thread.__init__(self)
         self.daemon = True  # kill thread when main killed (KeyboardInterrupt)
         self.was_killed = False
@@ -100,11 +102,12 @@ class TMonitor(Thread):
             # Then monitor!
             cur_t = self._time()
             # Check for each tqdm instance if one is waiting too long to print
-            for instance in self.tqdm_cls._instances:
+            # NB: copy avoids size change during iteration RuntimeError
+            for instance in self.tqdm_cls._instances.copy():
                 # Only if mininterval > 1 (else iterations are just slow)
                 # and last refresh was longer than maxinterval in this instance
                 if instance.miniters > 1 and \
-                  (cur_t - instance.last_print_t) >= instance.maxinterval:
+                   (cur_t - instance.last_print_t) >= instance.maxinterval:
                     # We force bypassing miniters on next iteration
                     # dynamic_miniters should adjust mininterval automatically
                     instance.miniters = 1
@@ -289,7 +292,7 @@ class tqdm(object):
                 '{0:3.0f}%|'.format(percentage)
             r_bar = '| {0}/{1} [{2}<{3}, {4}{5}]'.format(
                     n_fmt, total_fmt, elapsed_str, remaining_str, rate_fmt,
-                    ', '+postfix if postfix else '')
+                    ', ' + postfix if postfix else '')
 
             if ncols == 0:
                 return l_bar[:-1] + r_bar[1:]
@@ -314,7 +317,7 @@ class tqdm(object):
                             'l_bar': l_bar,
                             'r_bar': r_bar,
                             'desc': prefix if prefix else '',
-                            'postfix': ', '+postfix if postfix else '',
+                            'postfix': ', ' + postfix if postfix else '',
                             # 'bar': full_bar  # replaced by procedure below
                             }
 
@@ -365,7 +368,7 @@ class tqdm(object):
         else:
             return (prefix if prefix else '') + '{0}{1} [{2}, {3}{4}]'.format(
                 n_fmt, unit, elapsed_str, rate_fmt,
-                ', '+postfix if postfix else '')
+                ', ' + postfix if postfix else '')
 
     def __new__(cls, *args, **kwargs):
         # Create a new instance
@@ -406,17 +409,20 @@ class tqdm(object):
             # Kill monitor if no instances are left
             if not cls._instances and cls.monitor:
                 cls.monitor.exit()
-                del cls.monitor
+                try:
+                    del cls.monitor
+                except AttributeError:
+                    pass
                 cls.monitor = None
         except KeyError:
             pass
 
     @classmethod
-    def write(cls, s, file=sys.stdout, end="\n"):
+    def write(cls, s, file=None, end="\n"):
         """
         Print a message via tqdm (without overlap with bars)
         """
-        fp = file
+        fp = file if file is not None else sys.stdout
 
         # Clear all bars
         inst_cleared = []
@@ -434,7 +440,7 @@ class tqdm(object):
         # Force refresh display of bars we cleared
         for inst in inst_cleared:
             # Avoid race conditions by checking that the instance started
-            if hasattr(inst, 'start_t'):
+            if hasattr(inst, 'start_t'):  # pragma: nocover
                 inst.refresh()
         # TODO: make list of all instances incl. absolutely positioned ones?
 
@@ -542,7 +548,7 @@ class tqdm(object):
         GroupBy.progress_transform = inner_generator('transform')
 
     def __init__(self, iterable=None, desc=None, total=None, leave=True,
-                 file=sys.stderr, ncols=None, mininterval=0.1,
+                 file=None, ncols=None, mininterval=0.1,
                  maxinterval=10.0, miniters=None, ascii=None, disable=False,
                  unit='it', unit_scale=False, dynamic_ncols=False,
                  smoothing=0.3, bar_format=None, initial=0, position=None,
@@ -568,7 +574,7 @@ class tqdm(object):
             upon termination of iteration.
         file  : `io.TextIOWrapper` or `io.StringIO`, optional
             Specifies where to output the progress messages
-            [default: sys.stderr]. Uses `file.write(str)` and `file.flush()`
+            (default: sys.stderr). Uses `file.write(str)` and `file.flush()`
             methods.
         ncols  : int, optional
             The width of the entire output message. If specified,
@@ -594,7 +600,7 @@ class tqdm(object):
         ascii  : bool, optional
             If unspecified or False, use unicode (smooth blocks) to fill
             the meter. The fallback is to use ASCII characters `1-9 #`.
-        disable  : bool or None, optional
+        disable  : bool, optional
             Whether to disable the entire progressbar wrapper
             [default: False]. If set to None, disable on non-TTY.
         unit  : str, optional
@@ -648,6 +654,9 @@ class tqdm(object):
             self._instances.remove(self)
             return
 
+        if file is None:
+            file = sys.stderr
+
         if kwargs:
             self.disable = True
             self.pos = self._get_free_pos(self)
@@ -669,9 +678,16 @@ class tqdm(object):
                 dynamic_ncols:  # pragma: no cover
             if dynamic_ncols:
                 dynamic_ncols = _environ_cols_wrapper()
-                ncols = dynamic_ncols(file)
+                if dynamic_ncols:
+                    ncols = dynamic_ncols(file)
+                # elif ncols is not None:
+                #     ncols = 79
             else:
-                ncols = _environ_cols_wrapper()(file)
+                _dynamic_ncols = _environ_cols_wrapper()
+                if _dynamic_ncols:
+                    ncols = _dynamic_ncols(file)
+                # else:
+                #     ncols = 79
 
         if miniters is None:
             miniters = 0
@@ -726,7 +742,11 @@ class tqdm(object):
 
         # if nested, at initial sp() call we replace '\r' by '\n' to
         # not overwrite the outer progress bar
-        self.pos = self._get_free_pos(self) if position is None else position
+        if position is None:
+            self.pos = self._get_free_pos(self)
+        else:
+            self.pos = position
+            self._instances.remove(self)
 
         if not gui:
             # Initialize the screen printer
@@ -746,9 +766,10 @@ class tqdm(object):
         self.start_t = self.last_print_t
 
     def __len__(self):
-        return (self.iterable.shape[0] if hasattr(self.iterable, 'shape')
-                else len(self.iterable)) if self.iterable is not None \
-            else self.total
+        return self.total if self.iterable is None else \
+            (self.iterable.shape[0] if hasattr(self.iterable, "shape")
+             else len(self.iterable) if hasattr(self.iterable, "__len__")
+             else self.total)
 
     def __enter__(self):
         return self
@@ -763,7 +784,9 @@ class tqdm(object):
     def __repr__(self):
         return self.format_meter(self.n, self.total,
                                  self._time() - self.start_t,
-                                 self.ncols, self.desc, self.ascii, self.unit,
+                                 self.dynamic_ncols(self.fp)
+                                 if self.dynamic_ncols else self.ncols,
+                                 self.desc, self.ascii, self.unit,
                                  self.unit_scale, 1 / self.avg_time
                                  if self.avg_time else None, self.bar_format,
                                  self.postfix)
@@ -877,10 +900,10 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
                                 # EMA-weight miniters to converge
                                 # towards the timeframe of mininterval
                                 miniters = smoothing * delta_it * \
-                                              (mininterval / delta_t
-                                               if mininterval and delta_t
-                                               else 1) + \
-                                              (1 - smoothing) * miniters
+                                    (mininterval / delta_t
+                                     if mininterval and delta_t
+                                     else 1) + \
+                                    (1 - smoothing) * miniters
                             else:
                                 # Maximum nb of iterations between 2 prints
                                 miniters = max(miniters, delta_it)
@@ -930,7 +953,7 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
             if delta_t >= self.mininterval:
                 cur_t = self._time()
                 delta_it = self.n - self.last_print_n  # should be n?
-                elapsed = cur_t - self.start_t
+                # elapsed = cur_t - self.start_t
                 # EMA (not just overall average)
                 if self.smoothing and delta_t and delta_it:
                     self.avg_time = delta_t / delta_it \
@@ -947,13 +970,7 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
                     self.moveto(self.pos)
 
                 # Print bar's update
-                self.sp(self.format_meter(
-                    self.n, self.total, elapsed,
-                    (self.dynamic_ncols(self.fp) if self.dynamic_ncols
-                     else self.ncols),
-                    self.desc, self.ascii, self.unit, self.unit_scale,
-                    1 / self.avg_time if self.avg_time else None,
-                    self.bar_format, self.postfix))
+                self.sp(self.__repr__())
 
                 if self.pos:
                     self.moveto(-self.pos)
@@ -967,16 +984,16 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
                     if self.maxinterval and delta_t >= self.maxinterval:
                         if self.mininterval:
                             self.miniters = delta_it * self.mininterval \
-                                        / delta_t
+                                / delta_t
                         else:
                             self.miniters = delta_it * self.maxinterval \
-                                        / delta_t
+                                / delta_t
                     elif self.smoothing:
                         self.miniters = self.smoothing * delta_it * \
-                                        (self.mininterval / delta_t
-                                         if self.mininterval and delta_t
-                                         else 1) + \
-                                        (1 - self.smoothing) * self.miniters
+                            (self.mininterval / delta_t
+                             if self.mininterval and delta_t
+                             else 1) + \
+                            (1 - self.smoothing) * self.miniters
                     else:
                         self.miniters = max(self.miniters, delta_it)
 
